package eap.parfile.doc;

import eap.parfile.*;

import javax.xml.parsers.*;
import org.xml.sax.*;
import org.w3c.dom.*;
import javax.xml.transform.*;
import javax.xml.transform.dom.*;
import javax.xml.transform.stream.*;

import java.io.*;
import java.util.*;

/*************************************************************************
*
*************************************************************************/
public class ToolDoc {

Document document;
Parfile parfile;
Calendar date;

/*************************************************************************
*
*************************************************************************/
public ToolDoc(File file) throws ParserConfigurationException, 
                                                 SAXException,
                                                  IOException,
                                           ParameterException {
                                           
    /*****************************
    * get the last modified date *
    *****************************/
    date = Calendar.getInstance();
    date.setTimeInMillis(file.lastModified());

    /************************
    * read the XML document *
    ************************/
    DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
    DocumentBuilder builder = factory.newDocumentBuilder();
    document = builder.parse(file);


    /***************************
    * get the name of the tool *
    ***************************/
    String tool = getToolName();
    if(tool == null) {
        throw new ParameterException("Could not get name of tool");
    }

    /**************************
    * try to read the parfile *
    **************************/
    parfile = new Parfile(
              new BufferedReader(
              new FileReader(
              new File(file.getParent(), tool+".par") )));


    /*******************************************
    * make sure the XML file doesn't have any
    * parameters which aren't in the parfile
    ********************************************/
    Set xml_params =         getParameters();
    Set par_params = parfile.getParameters();
    
    xml_params.removeAll(par_params);
    
    if(xml_params.size() != 0 ) {
        throw new ParameterException("XML file has parameters not in the parfile: "+xml_params);
    }



} // end of constructor

/*************************************************************************
*
*************************************************************************/
public String getToolName() {

    try {
        return document.getElementsByTagName("TOOL")
                       .item(0)
                       .getFirstChild()
                       .getNodeValue();

    } catch(Exception e) {
        return null;
    }
} // end of getToolName method

/*************************************************************************
* return a set containing the names of all the parameters
*************************************************************************/
public Set getParameters() throws ParameterException {

    Set set = new HashSet();

    /***************************
    * find the parameters list *
    ***************************/
    NodeList list = document.getElementsByTagName("PARAM");

    for(int i=0; i<list.getLength(); ++i) {
        Element param = (Element)list.item(i);

        /*************************************************
        * get a list of NAME elements in the parameter
        * there should just be one
        *************************************************/
        NodeList names = param.getElementsByTagName("NAME");
        if(names.getLength() != 1) {
            throw new ParameterException("Could not get name for parameter "+i);
        }

        /***************
        * get the name *
        ***************/
        String name;
        try { name = names.item(0).getFirstChild().getNodeValue(); }
        catch(NullPointerException e) {
            throw new ParameterException("Could not get name for parameter "+i);
        }

        set.add(name);

    } // end of loop over parameters

    return set;


} // end of getParameters method

/*************************************************************************
*
*************************************************************************/
protected Element getParameterElement(String name) {

    /***************************
    * find the parameters list *
    ***************************/
    NodeList list = document.getElementsByTagName("PARAM");

    for(int i=0; i<list.getLength(); ++i) {
        Element param = (Element)list.item(i);

        /*************************************************
        * get a list of NAME elements in the parameter
        * there should just be one
        *************************************************/
        NodeList names = param.getElementsByTagName("NAME");
        if(names.getLength() != 1) {
            continue;
        }

        /***************
        * get the name *
        ***************/
        try {
            if( names.item(0).getFirstChild().getNodeValue().equals(name) ) {
                return param; 
            }
        } catch(NullPointerException e) {continue;}

    } // end of loop over parameters
    
    return null;

} // end of getParameterElement method




/*****************************************************************************
* adds entries for all the parameters which are in the parfile but not
* in the XML file. Returns true if some parameters were added.
*****************************************************************************/
public boolean addMissingParameters() throws ParameterException {

    /****************************************************************
    * get a set of all the parameters which are not in the XML file *
    ****************************************************************/
    Set xml_params =         getParameters();
    Set par_params = parfile.getParameters();

    par_params.removeAll(xml_params);
    par_params.remove("mode");
    
    /*******************************
    * check if anything is missing *
    *******************************/
    if(par_params.size() == 0) return false;
    
    /***************************************
    * loop over all the missing parameters *
    ***************************************/
    for(Iterator it=par_params.iterator(); it.hasNext(); ) {
        String name_string = (String)it.next();
        
        /***********************************
        * get information from the parfile *
        ***********************************/
        String prompt = parfile.get(name_string).getPrompt();

        /**************************
        * assemble the PARAM node *
        **************************/
        Element param = document.createElement("PARAM");
        Element name  = document.createElement("NAME");

        name.appendChild(document.createTextNode(name_string) );
        param.appendChild(name);
        
        param.appendChild(document.createTextNode(prompt));
        
        /****************************************
        * add the new parameter to the document *
        ****************************************/
        Node param_list = document.getElementsByTagName("PARAMETERS").item(0);
        param_list.appendChild(param);
        

    } // end of loop over missing parameters


    /**********************************************
    * return true to show that we added something *
    **********************************************/
    return true;

} // end of addMissingParameter method

/*************************************************************************
*
*************************************************************************/
public void addParameterInfo() throws ParameterException {

    /***************************
    * loop over the parameters *
    ***************************/
    for(Iterator it = getParameters().iterator(); it.hasNext(); ) {
        String name = (String)it.next();
        
        /*********************************************
        * get the objects representing the parameter *
        *********************************************/
        Parameter par_param = parfile.get(name);
        Element   xml_param = getParameterElement(name);

        /**********************************************
        * get the first node after the parameter name *
        **********************************************/
        Node node = xml_param.getFirstChild().getNextSibling();

        /************************
        * parameter type/format *
        ************************/
        Element element;
        element = document.createElement("FORMAT");
        element.appendChild(document.createTextNode(par_param.getType()));
        xml_param.insertBefore(element, node);

        /*******
        * mode *
        *******/
        element = document.createElement("MODE");
        element.appendChild(document.createTextNode(par_param.getMode()));
        xml_param.insertBefore(element, node);
        
        /*******
        * value *
        *******/
        element = document.createElement("VALUE");
        element.appendChild(document.createTextNode(par_param.getValue()));
        xml_param.insertBefore(element, node);
        
    } // end of loop over parameters



} // end of addParameterInfo method

/*************************************************************************
*
*************************************************************************/
public void timeStamp() throws ParameterException {

    /**********************************************************
    * assemble the date structure taking values from the
    * XML files last modified date that the constructor saved
    **********************************************************/
    Element element   = document.createElement("DATE");
    Element year      = document.createElement("YEAR");
    Element month     = document.createElement("MONTH");
    Element day       = document.createElement("DAY");
    Element hour      = document.createElement("HOUR");
    Element minute    = document.createElement("MINUTE");
    Element second    = document.createElement("SECOND");
    
    element.appendChild(year);
    element.appendChild(month);
    element.appendChild(day);
    element.appendChild(hour);
    element.appendChild(minute);
    element.appendChild(second);
    
     year.appendChild(document.createTextNode(""+ date.get(Calendar.YEAR)));
    month.appendChild(document.createTextNode(""+(date.get(Calendar.MONTH)+1)));
      day.appendChild(document.createTextNode(""+ date.get(Calendar.DAY_OF_MONTH)));
     hour.appendChild(document.createTextNode(""+ date.get(Calendar.HOUR_OF_DAY)));
   minute.appendChild(document.createTextNode(""+ date.get(Calendar.MINUTE)));
   second.appendChild(document.createTextNode(""+ date.get(Calendar.SECOND)));
   
   NodeList list = document.getElementsByTagName("HELPFILE");
   if(list.getLength() != 1) {
       throw new ParameterException("This is not a helpfile");
   }
   
   list.item(0).appendChild(element);
}

/*************************************************************************
*
*************************************************************************/
public void format(File stylesheet) throws TransformerConfigurationException,
                                                        TransformerException,
                                                                 IOException {
    /**********************
    * read the stylesheet *
    **********************/
    TransformerFactory tFactory = TransformerFactory.newInstance();
    StreamSource stylesource = new StreamSource(stylesheet);
    Transformer transformer = tFactory.newTransformer(stylesource);


    /*************************
    * transform the document *
    *************************/
    DOMSource source = new DOMSource(document);
    StreamResult result = new StreamResult(System.out);
    transformer.transform(source, result);

} // end of format method




/*************************************************************************
*
*************************************************************************/
public static void main (String args[]) throws Exception {

    ToolDoc doc = new ToolDoc(new File(args[0]));

    doc.addMissingParameters();
    doc.addParameterInfo();
    doc.timeStamp();
    
    doc.format(new File(args[1]));


} // end of main method

} // end of ToolDoc class
